Attribute VB_Name = "modTimeConversionFunctions"
Option Explicit
Option Compare Text
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' modConversionFunctions
' By Chip Pearson, www.cpearson.com, chip@cpearson.com
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' This module contains conversion functions used to convert FILETIMEs, SYSTEMTIMEs, and Serial Times,
' including functions for converting to/from Local Time To GMTTime.
'
' Note that the functions in this module call upon one another. You should import this entire
' module into your project rather than copy/pasting the individual functions.
'
' -------------------------
' Functions In This Module:
' -------------------------
'
' FileTimeToSerialTime
' --------------------
'   This function converts a FILETIME to a Serial Time. This function is the converse of
'   the SerialTimeToFileTime function.
'
' GMTSerialTimeToLocalSerialTime
' ------------------------------
'   This function converts a GMT serial time to the local serial time. This is the
'   converse of the LocalSerialTimeToGMTSerialTime function.
'
' GMTSystemTimeToLocalSystemTime
' ------------------------------
'   This function converts a SYSTEMTIME containing a GMT value to a SYSTEMTIME containing
'   the local time. This is the converse of the LocalSystemTimeToGMTSystemTime function.
'
' LocalFileTimeToGMTFileTime
' --------------------------
'   This function converts a FILETIME containing the local time to a FILETIME containing
'   the GMT Time. This is the converse of GMTFileTimeToLocalFileTime.
'
' LocalSerialTimeToGMTSerialTime
' ------------------------------
'   This function converts a Local serial time to the GMT serial time. This is the
'   converse of the GMTSerialTimeToLocalSerialTime function.
'
' LocalSystemTimeToGMTSystemTime
' ------------------------------
'   This funtion converts a SYSTEMTIME containing the local time to a SYSTEMTIME
'   containing the GMT Time. This is the converse of the GMTSystemTimeToLocalSystemTime
'   function.
'
' SerialTimeToFileTime
' --------------------
'   This function converts a Serial Time to a FILETIME. This function is the converse
'   of the FileTimeToSerialTime function.
'
' SerialTimeToSystemTime
' ----------------------
'   This function converts a Serial Time to a SYSTEMTIME.
'
' SystemTimeToSerialTime
' ----------------------
'   This function converts a SYSTEMTIME to a Serial time.
'
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'''''''''''''''''''''''''''''''''''
' used by FormatMessage
'''''''''''''''''''''''''''''''''''
Private Const FORMAT_MESSAGE_ALLOCATE_BUFFER = &H100
Private Const FORMAT_MESSAGE_ARGUMENT_ARRAY = &H2000
Private Const FORMAT_MESSAGE_FROM_HMODULE = &H800
Private Const FORMAT_MESSAGE_FROM_STRING = &H400
Private Const FORMAT_MESSAGE_FROM_SYSTEM = &H1000
Private Const FORMAT_MESSAGE_MAX_WIDTH_MASK = &HFF
Private Const FORMAT_MESSAGE_IGNORE_INSERTS = &H200
Private Const FORMAT_MESSAGE_TEXT_LEN = &HA0 ' from ERRORS.H C++ include file.

'''''''''''''''''''''''''''''''''''''''
' Windows API Functions
'''''''''''''''''''''''''''''''''''''''
Private Declare Function FormatMessage Lib "kernel32" Alias "FormatMessageA" ( _
    ByVal dwFlags As Long, _
    lpSource As Any, _
    ByVal dwMessageId As Long, _
    ByVal dwLanguageId As Long, _
    ByVal lpBuffer As String, _
    ByVal nSize As Long, _
    Arguments As Long) As Long

Private Declare Function LocalFileTimeToFileTime Lib "kernel32" _
    (lpLocalFileTime As FILETIME, _
     lpFileTime As FILETIME) As Long

Private Declare Function FileTimeToLocalFileTime Lib "kernel32" ( _
    lpFileTime As FILETIME, _
    lpLocalFileTime As FILETIME) As Long

Private Declare Function SystemTimeToFileTime Lib "kernel32" _
    (lpSystemTime As SYSTEMTIME, _
    lpFileTime As FILETIME) As Long

Private Declare Function CreateFile Lib "kernel32" Alias "CreateFileA" _
    (ByVal lpFilename As String, _
    ByVal dwDesiredAccess As Long, _
    ByVal dwShareMode As Long, _
    ByVal lpSecurityAttributes As Long, _
    ByVal dwCreationDisposition As Long, _
    ByVal dwFlagsAndAttributes As Long, _
    ByVal hTemplateFile As Long) As Long

Private Declare Function CloseHandle Lib "kernel32" _
   (ByVal hObject As Long) As Long

Private Declare Function FileTimeToSystemTime Lib "kernel32" ( _
    lpFileTime As FILETIME, _
    lpSystemTime As SYSTEMTIME) As Long

Private Declare Function GetTimeZoneInformation Lib "kernel32" ( _
    lpTimeZoneInformation As TIME_ZONE_INFORMATION) As Long


Public Function GMTFileTimeToLocalFileTime(GmtFileTime As FILETIME, ByRef LocalFileTime As FILETIME) As Boolean
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' GMTFileTimeToLocalFileTime
' This converts a FILETIME containing a GMT time value to a FILETIME containing
' the local file time. You pass in as the LocalFileTime a variable of type
' FILETIME which will receive the local file time.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim Res As Long
Dim ErrNum As Long
Dim ErrText As String

Res = FileTimeToLocalFileTime(lpFileTime:=GmtFileTime, lpLocalFileTime:=LocalFileTime)
If Res = 0 Then
    '''''''''''''''''''''
    ' An error occurred
    '''''''''''''''''''''
    ErrNum = Err.LastDllError
    ErrText = GetSystemErrorMessageText(ErrNum)
    Debug.Print "Error With FileTimeToLocalFileTime:" & vbCrLf & _
                "Err:  " & CStr(ErrNum) & vbCrLf & _
                "Desc: " & ErrText
    GMTFileTimeToLocalFileTime = False
    Exit Function
End If

GMTFileTimeToLocalFileTime = True

End Function

Public Function LocalFileTimeToGMTFileTime(LocalFileTime As FILETIME, GmtFileTime As FILETIME) As Boolean
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' LocalFileTimeToGMTFileTime
' This converts a local FILETIME to a GMT FILETIME. The value LocalFileTime
' is the local FILETIME to be converted. GMTFileTime is a variable in the
' calling process, of type FILETIME, in which the GMT FILETIME is to be
' stored.
' The function returns True if the conversion was successful, or
' False if a conversion error occurred.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim Res As Long
Dim ErrNum As Long
Dim ErrText As String

Res = LocalFileTimeToFileTime(lpLocalFileTime:=LocalFileTime, lpFileTime:=GmtFileTime)

If Res = 0 Then
    '''''''''''''''''''''
    ' An error occurred
    '''''''''''''''''''''
    ErrNum = Err.LastDllError
    ErrText = GetSystemErrorMessageText(ErrNum)
    Debug.Print "Error With LocalFileTimeToFileTime:" & vbCrLf & _
                "Err:  " & CStr(ErrNum) & vbCrLf & _
                "Desc: " & ErrText
    LocalFileTimeToGMTFileTime = False
    Exit Function
End If

LocalFileTimeToGMTFileTime = True


End Function

Public Function GMTSystemTimeToLocalSystemTime(GMTSystemTime As SYSTEMTIME, _
    LocalSystemTime As SYSTEMTIME) As Boolean
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' GMTSystemTimeToLocalSystemTime
' This function converts the GMT time stored in GMTSystemTime to a
' SYSTEMTIME structure containing the local time. The parameter
' LocalSystemTime is a variable in the calling procedure that
' will be populated with the converted time value.
' The function returns True if the conversion was successful, or
' False if a conversion error occurred.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Dim Res As Long
Dim ErrNum As Long
Dim ErrText As String
Dim LocalFileTime As FILETIME
Dim GmtFileTime As FILETIME

'''''''''''''''''''''''''''''''''''''''''''
' Convert GTMSystemTime to a FILETIME
' value.
'''''''''''''''''''''''''''''''''''''''''''
Res = SystemTimeToFileTime(lpSystemTime:=GMTSystemTime, lpFileTime:=GmtFileTime)
If Res = 0 Then
    '''''''''''''''''''''
    ' An error occurred
    '''''''''''''''''''''
    ErrNum = Err.LastDllError
    ErrText = GetSystemErrorMessageText(ErrNum)
    Debug.Print "Error With SystemTimeToFileTime:" & vbCrLf & _
                "Err:  " & CStr(ErrNum) & vbCrLf & _
                "Desc: " & ErrText
    GMTSystemTimeToLocalSystemTime = False
    Exit Function
End If

''''''''''''''''''''''''''''''''''''''''''''
' Convert the GMT FileTime in GMTFileTime
' to the local time LocalFileTime.
''''''''''''''''''''''''''''''''''''''''''''
Res = FileTimeToLocalFileTime(lpFileTime:=GmtFileTime, lpLocalFileTime:=LocalFileTime)
If Res = 0 Then
    '''''''''''''''''''''
    ' An error occurred
    '''''''''''''''''''''
    ErrNum = Err.LastDllError
    ErrText = GetSystemErrorMessageText(ErrNum)
    Debug.Print "Error With FileTimeToLocalFileTime:" & vbCrLf & _
                "Err:  " & CStr(ErrNum) & vbCrLf & _
                "Desc: " & ErrText
    GMTSystemTimeToLocalSystemTime = False
    Exit Function
End If
'''''''''''''''''''''''''''''''''''''''''''
' Convert the local LocalFileTime to
' a SYSTEMTIME.
'''''''''''''''''''''''''''''''''''''''''''
Res = FileTimeToSystemTime(lpFileTime:=LocalFileTime, lpSystemTime:=LocalSystemTime)
If Res = 0 Then
    '''''''''''''''''''''
    ' An error occurred
    '''''''''''''''''''''
    ErrNum = Err.LastDllError
    ErrText = GetSystemErrorMessageText(ErrNum)
    Debug.Print "Error With FileTimeToSystemTime:" & vbCrLf & _
                "Err:  " & CStr(ErrNum) & vbCrLf & _
                "Desc: " & ErrText
    GMTSystemTimeToLocalSystemTime = False
    Exit Function
End If

GMTSystemTimeToLocalSystemTime = True


End Function

Public Function LocalSystemTimeToGMTSystemTime(LocalSystemTime As SYSTEMTIME, _
    ByRef GMTSystemTime As SYSTEMTIME) As Boolean
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' LocalSystemTimeToGMTSystemTime
' This converts a SYSTEMTIME containing the local time to a SYSTEMTIME
' containing the GMT time.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim Res As Long
Dim ErrNum As Long
Dim ErrText As String
Dim LocalFileTime As FILETIME
Dim GmtFileTime As FILETIME

'''''''''''''''''''''''''''''''''''''''''''''
' Convert LocalSystemTime to a FILETIME
' containing the local time.
'''''''''''''''''''''''''''''''''''''''''''''
Res = SystemTimeToFileTime(lpSystemTime:=LocalSystemTime, lpFileTime:=LocalFileTime)
If Res = 0 Then
    '''''''''''''''''''''
    ' An error occurred
    '''''''''''''''''''''
    ErrNum = Err.LastDllError
    ErrText = GetSystemErrorMessageText(ErrNum)
    Debug.Print "Error With SystemTimeToFileTime:" & vbCrLf & _
                "Err:  " & CStr(ErrNum) & vbCrLf & _
                "Desc: " & ErrText
    LocalSystemTimeToGMTSystemTime = False
    Exit Function
End If
''''''''''''''''''''''''''''''''''''''''''''''''
' Convert LocalFileTime to GMTFileTime
''''''''''''''''''''''''''''''''''''''''''''''''
Res = LocalFileTimeToFileTime(lpLocalFileTime:=LocalFileTime, lpFileTime:=GmtFileTime)
If Res = 0 Then
    '''''''''''''''''''''
    ' An error occurred
    '''''''''''''''''''''
    ErrNum = Err.LastDllError
    ErrText = GetSystemErrorMessageText(ErrNum)
    Debug.Print "Error With LocalFileTimeToFileTime:" & vbCrLf & _
                "Err:  " & CStr(ErrNum) & vbCrLf & _
                "Desc: " & ErrText
    LocalSystemTimeToGMTSystemTime = False
    Exit Function
End If
''''''''''''''''''''''''''''''''''''''''''''''''
' Convert the GMTFileTime to a SYSTEMTIME.
''''''''''''''''''''''''''''''''''''''''''''''''
Res = FileTimeToSystemTime(lpFileTime:=GmtFileTime, lpSystemTime:=GMTSystemTime)
If Res = 0 Then
    '''''''''''''''''''''
    ' An error occurred
    '''''''''''''''''''''
    ErrNum = Err.LastDllError
    ErrText = GetSystemErrorMessageText(ErrNum)
    Debug.Print "Error With FileTimeToSystemTime:" & vbCrLf & _
                "Err:  " & CStr(ErrNum) & vbCrLf & _
                "Desc: " & ErrText
    LocalSystemTimeToGMTSystemTime = False
    Exit Function
End If

LocalSystemTimeToGMTSystemTime = True

End Function
    
Public Function SystemTimeToSerialTime(SysTime As SYSTEMTIME) As Date
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SystemTimeToSerialTime
' This function converts a SYSTEMTIME to a Double Serial DateTime
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
With SysTime
    SystemTimeToSerialTime = DateSerial(.wYear, .wMonth, .wDay) + _
                        TimeSerial(.wHour, .wMinute, .wSecond)
End With

End Function

Public Function FileTimeToSerialTime(FileTimeValue As FILETIME) As Date
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' FileTimeToSerialTime
' This function converts a FILETIME to a Double Serial DateTime.
' TESTED
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Dim SysTime As SYSTEMTIME
Dim Res As Long
Dim ErrNum As Long
Dim ErrText As String
Dim ResultDate As Date

'''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Convert FileTimeValue FILETIME to SysTime SYSTEMTIME.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''
Res = FileTimeToSystemTime(lpFileTime:=FileTimeValue, lpSystemTime:=SysTime)
If Res = 0 Then
    '''''''''''''''''''''
    ' An error occurred
    '''''''''''''''''''''
    ErrNum = Err.LastDllError
    ErrText = GetSystemErrorMessageText(ErrNum)
    Debug.Print "Error With FileTimeToSystemTime:" & vbCrLf & _
                "Err:  " & CStr(ErrNum) & vbCrLf & _
                "Desc: " & ErrText
    FileTimeToSerialTime = False
    Exit Function
End If

With SysTime
    ResultDate = DateSerial(.wYear, .wMonth, .wDay) + _
                TimeSerial(.wHour, .wMinute, .wSecond)
End With

FileTimeToSerialTime = ResultDate


End Function

Public Function SerialTimeToFileTime(SerialTime As Date, ByRef FileTimeValue As FILETIME) As Boolean
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SerialTimeToFileTime
' This function converts a serial time to a FILETIME.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim SysTime As SYSTEMTIME
Dim FTime As FILETIME
Dim Res As Long
Dim ErrNum As Long
Dim ErrText As String

''''''''''''''''''''''''''''''''''''''''''''
' Load up SysTime with the values from
' SerialTime.
''''''''''''''''''''''''''''''''''''''''''''
With SysTime
    .wYear = Year(SerialTime)
    .wMonth = Month(SerialTime)
    .wDay = Day(SerialTime)
    .wDayOfWeek = Weekday(SerialTime) - 1
    .wHour = Hour(SerialTime)
    .wMinute = Minute(SerialTime)
    .wSecond = Second(SerialTime)
    .wMilliseconds = 0
End With

'''''''''''''''''''''''''''''''''''''''''
' Convert the SystemTime to a FileTime
'''''''''''''''''''''''''''''''''''''''''
Res = SystemTimeToFileTime(lpSystemTime:=SysTime, lpFileTime:=FileTimeValue)
If Res = 0 Then
    '''''''''''''''''''''
    ' An error occurred
    '''''''''''''''''''''
    ErrNum = Err.LastDllError
    ErrText = GetSystemErrorMessageText(ErrNum)
    Debug.Print "Error With SystemTimeToFileTime:" & vbCrLf & _
                "Err:  " & CStr(ErrNum) & vbCrLf & _
                "Desc: " & ErrText
    SerialTimeToFileTime = False
    Exit Function
End If
SerialTimeToFileTime = True

End Function

Public Function SerialTimeToSystemTime(SerialTime As Date, SysTime As SYSTEMTIME) As Boolean
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SerialTimeToSystemTime
' This function converts a serial date/time to a SYSTEMTIME.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

With SysTime
    .wYear = Year(SerialTime)
    .wMonth = Month(SerialTime)
    .wDay = Day(SerialTime)
    .wDayOfWeek = Weekday(SerialTime) - 1
    .wHour = Hour(SerialTime)
    .wMinute = Minute(SerialTime)
    .wSecond = Second(SerialTime)
    .wMilliseconds = 0
End With

SerialTimeToSystemTime = True

End Function

Public Function LocalSerialTimeToGMTSerialTime(LocalSerialTime As Date) As Date
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' LocalSerialTimeToGMTSerialTime
' This function converts LocalSerialTime to a GMT Serial Time
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim TZI As TIME_ZONE_INFORMATION
Dim Res As Long
Res = GetTimeZoneInformation(TZI)
''''''''''''''''''''''''''''''''''''''''''''''''
' The arithatic used to convert between
' local and GMT times is
'   GMT = LocalTime + Bias
' where Bias is taken from TZI. The Bias in TZI
' is the number of minutes to be added to LocalTime
' to get the GMT Time.
'   LocalTime = GMT - Bias
' Since Bias is the number of minutes
' between Local and GMT, we get
'  GMT = LocalTime + TimeSerial(0,Bias,0)
' or
'  LocalTime = GMT - TimeSerial(0,Bias,0)
''''''''''''''''''''''''''''''''''''''''''''''''
    LocalSerialTimeToGMTSerialTime = LocalSerialTime + TimeSerial(0, TZI.Bias, 0)
End Function

Public Function GMTSerialTimeToLocalSerialTime(GMTSerialTime As Date) As Date
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' GMTSerialTimeToLocalSerialTime
' This function converts a GMT Serial date to a local serial date.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim TZI As TIME_ZONE_INFORMATION
Dim Res As Long
Res = GetTimeZoneInformation(TZI)
''''''''''''''''''''''''''''''''''''''''''''''''
' The arithatic used to convert between
' local and GMT times is
'   GMT = LocalTime + Bias
' where Bias is taken from TZI. The Bias in TZI
' is the number of minutes to be added to LocalTime
' to get the GMT Time.
'   LocalTime = GMT - Bias
' Since Bias is the number of minutes
' between Local and GMT, we get
'  GMT = LocalTime + TimeSerial(0,Bias,0)
' or
'  LocalTime = GMT - TimeSerial(0,Bias,0)
''''''''''''''''''''''''''''''''''''''''''''''''
GMTSerialTimeToLocalSerialTime = GMTSerialTime - TimeSerial(0, TZI.Bias, 0)


End Function

